#include "ping.h"
#include "bsp.h"

#if 1
#define WIZ_PING_DBG(x, ...) printf("[PING: DBG]" x "\r\n", ##__VA_ARGS__);
#define WIZ_PING_INFO(x, ...) printf("[PING: INFO]" x "\r\n", ##__VA_ARGS__);
#else
#define WIZ_PING_DBG(x, ...)
#endif

PINGMSGR PingRequest;	 // Variable for Ping Request
PINGMSGR PingReply;	     // Variable for Ping Reply
static uint16_t RandomID = 0x8888; 
static uint16_t RandomSeqNum = 0x7777;
uint8_t ping_reply_received = 0; 
uint8_t req_cnt=0;
uint8_t resp_cnt=0;

uint8_t ping_auto(uint8_t sn, uint8_t *addr)
{
    uint8_t i;
    uint32_t len = 0;
    uint8_t cnt = 0;

    for(i=0; i<=3; i++)
    {
        switch(getSn_SR(sn))
        {
            case SOCK_CLOSED:
                close(sn);
                WIZ_PING_INFO("Open %d Socket\r\n",sn);

            
                //WIZCHIP_WRITE(Sn_PROTO(sn),IPPROTO_ICMP);  // set ICMP Protocol
                //setSn_DPORT(Sn_PROTO(sn), IPPROTO_ICMP);
                //WIZCHIP_WRITE(WZ_SN_PNR(sn),IPPROTO_ICMP);  // set ICMP Protocol
                //setSn_PROTO(sn, IPPROTO_ICMP);
				WIZCHIP_WRITE(Sn_PROTO(sn), IPPROTO_ICMP);              // set ICMP Protocol
                //WIZCHIP_WRITE(Sn_TOS(sn), IPPROTO_ICMP);
                if(socket(sn, Sn_MR_IPRAW, 3000, 0) != sn)      // open the SOCKET with IPRAW mode, if fail then Error
                {
                    WIZ_PING_INFO("socket %d fail",sn);
                    return SOCKET_ERROR;
                }
                /* Check socket register. */
                while(getSn_SR(sn) != SOCK_IPRAW);
                WIZ_PING_DBG("%d sock, SOCK_CLOSED, Sn_SR : %x\r\n", sn, getSn_SR(sn));
            

                delay_ms(1000); // wait 1s
                delay_ms(1000); // wait 1s
                break;
            case SOCK_IPRAW:
                ping_request(sn, addr);
                req_cnt++;
                while(1){
                    if((len = getSn_RX_RSR(sn) ) > 0)
                    {
                        ping_reply(sn, addr, len);
                        resp_cnt++;
                        break;
                    }
                    else if(cnt > 100)
                    {
                        WIZ_PING_INFO("Requeset Time out.");
                        cnt = 0;
                        break;
                    }
                    else
                    {
                        cnt++;
                        delay_ms(50);   // wait 1s
                    }
                }
                break;
            default:
                WIZ_PING_DBG("%d sock, default, Sn_SR : %x\r\n", sn, getSn_SR(sn));
                break;
        }
    }
    if(req_cnt >= 3)
    {
        WIZ_PING_INFO("Ping Requeset = %d, Ping Reply = %d",req_cnt,resp_cnt);
        close(sn);
        if(req_cnt == resp_cnt) return SUCCESS;
        else                    return REPLY_ERROR;
    }
    
    return REPLY_ERROR;
}

uint8_t ping_request(uint8_t sn, uint8_t *addr)
{
    uint16_t i;
    uint8_t src_addr[16];
    uint32_t src_addr_ptr;
    int32_t ret;

  	//Initailize flag for ping reply
    ping_reply_received = 0;

    /* make header of the ping-request  */
    PingRequest.Type = PING_REQUEST;    // Ping-Request
    PingRequest.Code = CODE_ZERO;               // Always '0'
    PingRequest.ID = htons(RandomID++);         // set ping-request's ID to random integer value
    PingRequest.SeqNum =htons(RandomSeqNum++);  // set ping-request's sequence number to ramdom integer value
    //size = 32;                                // set Data size

    /* Fill in Data[]  as size of BIF_LEN (Default = 32)*/
    for(i = 0 ; i < BUF_LEN; i++){	                                
        PingRequest.Data[i] = (i) % 8;		  //'0'~'8' number into ping-request's data 	
    }

     /* Do checksum of Ping Request */
    PingRequest.CheckSum = 0;   // value of checksum before calucating checksum of ping-request packet
    PingRequest.CheckSum = htons(checksum((uint8_t*)&PingRequest,sizeof(PingRequest)));  // Calculate checksum

     /* sendto ping_request to destination */
    ret = sendto(sn,(uint8_t *)&PingRequest,sizeof(PingRequest),addr,3000);// Send Ping-Request to the specified peer.
    if(ret<=0){  
        WIZ_PING_INFO("Fail to send ping-reply packet, error : %ld", ret);
    } else {
        WIZ_PING_INFO("Send Ping Request  to Destination (%d.%d.%d.%d)",
                (addr[0]), (addr[1]), (addr[2]), (addr[3]));
    }

    WIZ_PING_DBG("ID:%x SeqNum:%d Checksum:%x", htons(PingRequest.ID), 
                                                htons(PingRequest.SeqNum),
                                                htons(PingRequest.CheckSum));


	return 0;
} // ping request    

uint8_t ping_reply(uint8_t sn, uint8_t *addr,  uint32_t rlen){
	 
	 uint16_t tmp_checksum;	
	 uint16_t len;
	 uint16_t i;
	 uint8_t data_buf[128];
	 uint32_t port = 3000;

	 PINGMSGR PingReply;
    /* receive data from a destination */
    len = recvfrom(sn, (uint8_t *)data_buf,rlen,addr,&port);
    if(data_buf[0] == PING_REPLY || data_buf[0] == PING_REPLY_V6) 
    {
        PingReply.Type 		 = data_buf[0];
        PingReply.Code 		 = data_buf[1];
        PingReply.CheckSum   = (data_buf[3]<<8) + data_buf[2];
        PingReply.ID 		 = (data_buf[5]<<8) + data_buf[4];
        PingReply.SeqNum 	 = (data_buf[7]<<8) + data_buf[6];
        
        for(i=0; i<len-8 ; i++)
        {
           	PingReply.Data[i] = data_buf[8+i];
        }
        	/* check Checksum of Ping Reply */
        tmp_checksum = ~checksum(data_buf,(uint16_t)len);
        if(tmp_checksum != 0xffff)
        {
            WIZ_PING_INFO("tmp_checksum = %x",tmp_checksum);
        }
        else
        {
            /*  Output the Destination IP and the size of the Ping Reply Message*/
            WIZ_PING_INFO("Reply from %d.%d.%d.%d",(addr[0]),  (addr[1]),  (addr[2]),  (addr[3]));
            WIZ_PING_INFO("ID : %x, SeqNum : %x,  Data size : %d bytes",htons(PingReply.ID),htons(PingReply.SeqNum),(rlen+6) );
    
            /*  SET ping_reply_receiver to '1' and go out the while_loop (waitting for ping reply)*/
            ping_reply_received =1;
        }
    }
    else if(data_buf[0] == PING_REQUEST)
    {
        PingReply.Code 	 = data_buf[1];
        PingReply.Type 	 = data_buf[2];
        PingReply.CheckSum  = (data_buf[3]<<8) + data_buf[2];
        PingReply.ID 		 = (data_buf[5]<<8) + data_buf[4];
        PingReply.SeqNum 	 = (data_buf[7]<<8) + data_buf[6];
        
        for(i=0; i<len-8 ; i++)
        {
           	PingReply.Data[i] = data_buf[8+i];
        }
        	/* check Checksum of Ping Reply */
        tmp_checksum = PingReply.CheckSum;
        PingReply.CheckSum = 0;
        PingReply.CheckSum = htons(checksum((uint8_t*)&PingReply,len));
    
        if(tmp_checksum != PingReply.CheckSum){
            WIZ_PING_INFO("CheckSum is in correct %x shold be %x",   (tmp_checksum),  htons(PingReply.CheckSum)) ;
    	}else{
    		WIZ_PING_INFO("Checksum is correct") ;					
    	}
    
    	/*  Output the Destination IP and the size of the Ping Reply Message*/
        WIZ_PING_INFO("Request from %d.%d.%d.%d  ID:%x SeqNum:%x  :data size %d bytes\r\n",
                (addr[0]),  (addr[1]),  (addr[2]),  (addr[3]),  (PingReply.ID),  (PingReply.SeqNum),  (rlen+6) );
    	/*  SET ping_reply_receiver to '1' and go out the while_loop (waitting for ping reply)*/		   
    	ping_reply_received =1;
	}
    else{      
        WIZ_PING_INFO(" Unkonwn msg.");
    }
    
    return 0;
}// ping_reply

uint16_t checksum(uint8_t *data_buf, uint16_t len)
{
    uint16_t sum, tsum, i, j;
    uint32_t lsum;

    j = len >> 1;   // j = len / 2
    lsum = 0;
    tsum = 0;
    for(i=0; i<j; i++)
    {
        tsum = data_buf[i*2];
        tsum = tsum << 8;
        tsum += data_buf[i*2 +1];
        lsum += tsum;
        //printf("%.2x%.2x,",data_buf[i*2],data_buf[i*2 +1]);
    }
    if(len % 2)      // if len is odd
    {
        tsum = data_buf[i*2];
        lsum += (tsum << 8);
    }
    sum = (uint16_t)lsum;
    sum = ~(sum + (lsum>>16));

    return sum;
}

uint16_t htons(uint16_t hostshort)
{
    uint16_t netshort=0;
    netshort = (hostshort & 0xFF) << 8;
    netshort |= ((hostshort >> 8)&0xFF);
    return netshort;
}

